<?php

if (!interface_exists('IViewer'))
	throw new Exception('IViewer: FileViewer: no interface found.');
if (!class_exists('GtkVBox'))
	throw new Exception('IViewer: FileViewer: no parent class found.');

class FileViewer extends GtkVBox implements IViewer {
	private $env;
	private $actual_fs;
	private $model;
	private $tree;
	private $path;
	private $entryAddr;
	private $pathBar;
	private $side;
	private $langDialogWords;

	public function __construct(&$env) {
		parent::__construct();
		$this->env = &$env;

		/* create list view */
		$this->model = new GtkListStore(Gtk::TYPE_STRING, Gtk::TYPE_STRING, Gtk::TYPE_STRING, Gtk::TYPE_STRING);
		$this->tree = new GtkTreeView($this->model);
		$this->tree->set_search_column(0);
		$this->tree->set_enable_search(true);
		$this->tree->set_hover_selection(false);
		$this->tree->get_selection()->set_mode(Gtk::SELECTION_MULTIPLE);

		$cr1 = new GtkCellRendererText();
		$cr2 = new GtkCellRendererText();
		$cr3 = new GtkCellRendererText();
		$cr4 = new GtkCellRendererText();

		$clmnName = new GtkTreeViewColumn('Jméno');
		$clmnExt = new GtkTreeViewColumn('Přípona');
		$clmnSize = new GtkTreeViewColumn('Velikost');
		$clmnDate = new GtkTreeViewColumn('Datum');

		$clmnName->pack_start($cr1, true);
		$clmnExt->pack_start($cr2, true);
		$clmnSize->pack_start($cr3, true);
		$clmnDate->pack_start($cr4, true);

		$clmnName->set_attributes($cr1, 'text', 0);
		$clmnExt->set_attributes($cr2, 'text', 1);
		$clmnSize->set_attributes($cr3, 'text', 2);
		$clmnDate->set_attributes($cr4, 'text', 3);

		$this->tree->append_column($clmnName);
		$this->tree->append_column($clmnExt);
		$this->tree->append_column($clmnSize);
		$this->tree->append_column($clmnDate);

		$this->tree->connect_simple('row-activated', 'object_callback', $this, 'itemActivated');

		$scrl = new GtkScrolledWindow();
		$scrl->set_property('hscrollbar-policy', Gtk::POLICY_AUTOMATIC);
		$scrl->set_property('vscrollbar-policy', Gtk::POLICY_AUTOMATIC);
		$scrl->add_with_viewport($this->tree);

		$this->add($scrl);
	}

	public function init($side) {
		
		$this->side = $side;
		$this->entryAddr = ($side == 'left')?$this->env['gui']['entryAddrLeft']:$this->env['gui']['entryAddrRight'];
		$this->pathBar = ($side == 'left')?$this->env['gui']['pathBarLeft']:$this->env['gui']['pathBarRight'];

		$this->tree->connect('key_press_event', Array('FileViewer','callBackKeypressed'), $this, $this->env, $side);

		/* try to get some FS module */
		if (count($this->env['modules']['modulesFS']) < 1)
			throw new Exception($this->env['modules']['translationModule']->translate('MFileViewer_mfs_notfound'));
		$this->actual_fs = $this->env['modules']['modulesFS'][0];
		$drivers = $this->actual_fs->getDrivers();
		$separator = $this->actual_fs->getSeparator();
		if ($drivers) {
			foreach($drivers as $driver) {
				$button = new GtkToolButton(NULL,$driver.$separator);
				$this->pathBar->insert($button, -1);
				$button->connect_simple('clicked', Array('CMenuEvents','driverButtonClicked'), $this, $driver);
			}
		}
		$this->pathBar->show_all();

		$this->path = Array($drivers[0]);

		$this->refresh();
	}

	public function refresh() {
		$this->model->clear();
		$path = $this->computePath();

		$this->entryAddr->set_text($path);

		if (count($this->path) > 1)
			$this->model->append(array('..', '', '[DIR]', ''));

		$dirs = $this->actual_fs->getDirs($path);
		if ($dirs) {
			foreach ($dirs as $dir) {
				$this->model->append(array(iconv('ISO-8859-2', 'UTF-8', $dir), '', '[DIR]', ''));
				// nejake datum? -> nutno doplnit do modulu filesystemu
			}
		}
		
		$files = $this->actual_fs->getFiles($path);
		if ($files) {
			foreach ($files as $file) {
				$fileInfo = $this->actual_fs->getFileInfo($path.$file);
				$dot_pos = strrpos($file, '.');
				if (($dot_pos === false) || ($dot_pos == 0) || ($dot_pos == (strlen($file)-1))) {
					$file_name = $file;
					$file_ext = '';
				} else {
					$file_name = substr($file, 0, $dot_pos);
					$file_ext = substr($file, $dot_pos-strlen($file)+1);
				}
				$this->model->append(array(iconv('ISO-8859-2', 'UTF-8', $file_name), iconv('ISO-8859-2', 'UTF-8', $file_ext), $fileInfo['size'], Date('d.m.Y H:i:s',$fileInfo['createDate'])));
			}
		}
		
		$this->refreshLanguage();
	}
	
	private function refreshLanguage(){
		$this->tree->get_column(0)->set_title($this->env['modules']['translationModule']->translate('MFileViewer_clmn_name'));
		$this->tree->get_column(1)->set_title($this->env['modules']['translationModule']->translate('MFileViewer_clmn_ext'));
		$this->tree->get_column(2)->set_title($this->env['modules']['translationModule']->translate('MFileViewer_clmn_size'));
		$this->tree->get_column(3)->set_title($this->env['modules']['translationModule']->translate('MFileViewer_clmn_date'));
	}
	
	public function getDirPath() {
		return $this->computePath();
	}
	
	public function setDirPath($newDir) {
		$this->path = explode($this->actual_fs->getSeparator(),$newDir);
		$this->refresh();
	}
	
	public function getFileSelected() {
		$selected = $this->getFileSelection();
		if (count($selected) > 0)
			return $selected[0];
		else
			return false;
	}
	
	public function getFileSelection() {
		$result = Array();
		$selection = $this->tree->get_selection();
		list($model, $paths) = $selection->get_selected_rows();
		if ($paths) {
			foreach ($paths as $path) {
				$file = $model->get_value($model->get_iter($path), 0);
				$ext = $model->get_value($model->get_iter($path), 1);
				$item = $file;
				if ($ext != '')
					$item .= '.'.$ext;
				$result[] = $item;
			}
		}
		return $result;
	}
	
	public function getWorkingFileSelected() {
		$item = $this->getFileSelected();
		if ($item === false)
			return false;
		$path = $this->computePath();
		$path .= $item;
		return $this->actual_fs->getWorkingPath($path);
	}
	
	public function getWorkingFileSelection() {
		$result = Array();
		$path = $this->computePath();
		$items = $this->getFileSelection();
		if ($items) {
			foreach ($items as $item)		
				$result[] = $this->actual_fs->getWorkingPath($path.$item);
		}
		return $result;
	}

	private function computePath() {
		$result = implode($this->actual_fs->getSeparator(), $this->path);
		$result .= $this->actual_fs->getSeparator();
		return $result;
	}

	public function isActive() {
		return $this->tree->is_focus();
	}

	public function setActive() {
		$this->child_focus(Gtk::DIR_TAB_FORWARD);
	}

	public function itemActivated() {
		$item = $this->getFileSelected();
		$realItem = $this->getWorkingFileSelected();

		$path = $this->getDirPath();
		$path .= $item;
		if ($this->actual_fs->isDir($path)) {
			if ($item == '..')
				array_pop($this->path);
			else
				array_push($this->path, $item);
			$this->refresh();
		} else {
			if($this->env['os'] == 'unix')
				exec('run-mailcap "'.str_replace(array('\\','$','"'), array('\\\\','\$','\"'), $realItem).'" &');
			else
				exec('"'.$realItem.'"');
		}
	}

	public static function callBackKeypressed($widget, $event, $module, $env, $side) {
		switch($event->keyval) {
			case Gdk::KEY_F2:
				$module->refresh();
				break;
			case Gdk::KEY_F3:
				$env['modules']['events']->toolBarView();
				break;
			case Gdk::KEY_F4:
				$env['modules']['events']->toolBarEdit();
				break;
			case Gdk::KEY_F5:
				$env['modules']['events']->toolBarCopy();
				break;
			case Gdk::KEY_F6:
				if ($event->state == Gdk::SHIFT_MASK)
					$module->renameSelected();
				else
					$env['modules']['events']->toolBarMove();
				break;
			case Gdk::KEY_F7:
				if ($event->state == Gdk::MOD1_MASK)
					$env['modules']['events']->menuCommandsSearch();
				else
					$env['modules']['events']->toolBarMkdir();
				break;
			case Gdk::KEY_Delete:
			case Gdk::KEY_F8:
				$env['modules']['events']->toolBarDelete();
				break;
			case Gdk::KEY_F9:
			case Gdk::KEY_Tab:
				if ($side == 'left')
					$env['modules']['moduleViewRight']->setActive();
				else
					$env['modules']['moduleViewLeft']->setActive();
				return true;
				break;
			case Gdk::KEY_p:
				if ($event->state == Gdk::CONTROL_MASK)
					$env['modules']['events']->menuCommandsPrintDir();
				break;
		}
	}

	public function getActualFSModule() {
		return $this->actual_fs;
	}

	private function renameSelected() {
		$filename = $this->getFileSelected();
		$file = $this->getWorkingFileSelected();
		$newfile = $this->getDirPath();
		
		$dialog = new GtkMessageDialog(
			$this->env['gui']['window'],
			0,
			Gtk::MESSAGE_QUESTION,
			Gtk::BUTTONS_OK_CANCEL,
			$this->env['modules']['translationModule']->translate('MFileViewer_rename_msgbox')
		);
		$dialog->get_child()->pack_end(new GtkEntry($filename));
		$dialog->show_all();
		
		$result = $dialog->run();

		$children = $dialog->get_child()->get_children();
		array_pop($children);
		$myEntry = array_pop($children);

		$newfilename = $myEntry->get_text();
		$newfile .= $newfilename;

		if ($result == Gtk::RESPONSE_OK) {
			$this->actual_fs->move($newfile, $file);
			$this->refresh();
		}

		$dialog->destroy();
	}

}

?>
